@@ -1,7 +1,7 @@
 use numpy::{PyReadonlyArray1, PyReadonlyArray2, PyReadonlyArray3, PyReadonlyArrayDyn, ToPyArray};
 use pyo3::basic::CompareOp;
+use pyo3::prelude::*;
 use pyo3::types::PyType;
-use pyo3::{prelude::*, PyMappingProtocol, PyObjectProtocol};
 
 use crate::monomorphs::RaggedBufferI64;
 use crate::ragged_buffer_view::RaggedBufferView;
@@ -68,7 +68,7 @@ impl RaggedBufferF32 {
     fn size0(&self) -> usize {
         self.0.size0()
     }
-    fn size1(&self, py: Python, i: Option<usize>) -> PyResult<PyObject> {
+    fn size1(&mut self, py: Python, i: Option<usize>) -> PyResult<PyObject> {
         match i {
             Some(i) => self.0.size1(i).map(|s| s.into_py(py)),
             None => self.0.lengths(py).map(|ok| ok.into_py(py)),
@@ -77,10 +77,10 @@ impl RaggedBufferF32 {
     fn size2(&self) -> usize {
         self.0.size2()
     }
-    fn indices(&self, dim: usize) -> PyResult<RaggedBufferI64> {
+    fn indices(&mut self, dim: usize) -> PyResult<RaggedBufferI64> {
         Ok(RaggedBufferI64(self.0.indices(dim)?))
     }
-    fn flat_indices(&self) -> PyResult<RaggedBufferI64> {
+    fn flat_indices(&mut self) -> PyResult<RaggedBufferI64> {
         Ok(RaggedBufferI64(self.0.flat_indices()?))
     }
     #[classmethod]
@@ -91,7 +91,7 @@ impl RaggedBufferF32 {
         )?))
     }
     #[allow(clippy::type_complexity)]
-    fn padpack<'a>(&self, py: Python<'a>) -> PadpackResult<'a> {
+    fn padpack<'a>(&mut self, py: Python<'a>) -> PadpackResult<'a> {
         match self.0.padpack()? {
             Some((padbpack_index, padpack_batch, padpack_inverse_index, dims)) => Ok(Some((
                 padbpack_index.to_pyarray(py).reshape(dims)?,
@@ -103,16 +103,15 @@ impl RaggedBufferF32 {
             _ => Ok(None),
         }
     }
-    fn items(&self) -> PyResult<usize> {
+    fn items(&mut self) -> PyResult<usize> {
         self.0.items()
     }
     fn clone(&self) -> Self {
         RaggedBufferF32(self.0.deepclone())
     }
-}
-
-#[pyproto]
-impl PyObjectProtocol for RaggedBufferF32 {
+    fn materialize(&self) -> Self {
+        RaggedBufferF32(self.0.materialize())
+    }
     fn __str__(&self) -> PyResult<String> {
         self.0.__str__()
     }
@@ -130,19 +129,13 @@ impl PyObjectProtocol for RaggedBufferF32 {
             )),
         }
     }
-}
 
-#[derive(FromPyObject)]
-pub enum RaggedBufferF32OrF32 {
-    RB(RaggedBufferF32),
-    Scalar(f32),
-}
-
-// TODO: pass by reference?
-#[cfg(all())]
-#[pyproto]
-impl pyo3::PyNumberProtocol for RaggedBufferF32 {
-    fn __add__(lhs: RaggedBufferF32, rhs: RaggedBufferF32OrF32) -> PyResult<RaggedBufferF32> {
+    // Is substituted for #[cfg(any())] for bool.rs to omit method
+    #[cfg(all())]
+    fn __add__(
+        lhs: PyRef<RaggedBufferF32>,
+        rhs: RaggedBufferF32OrF32,
+    ) -> PyResult<RaggedBufferF32> {
         match rhs {
             RaggedBufferF32OrF32::RB(rhs) => Ok(RaggedBufferF32(
                 lhs.0.binop::<crate::ragged_buffer::BinOpAdd>(&rhs.0)?,
@@ -152,7 +145,12 @@ impl pyo3::PyNumberProtocol for RaggedBufferF32 {
             )),
         }
     }
-    fn __mul__(lhs: RaggedBufferF32, rhs: RaggedBufferF32OrF32) -> PyResult<RaggedBufferF32> {
+
+    #[cfg(all())]
+    fn __mul__(
+        lhs: PyRef<RaggedBufferF32>,
+        rhs: RaggedBufferF32OrF32,
+    ) -> PyResult<RaggedBufferF32> {
         match rhs {
             RaggedBufferF32OrF32::RB(rhs) => Ok(RaggedBufferF32(
                 lhs.0.binop::<crate::ragged_buffer::BinOpMul>(&rhs.0)?,
@@ -163,14 +161,12 @@ impl pyo3::PyNumberProtocol for RaggedBufferF32 {
         }
     }
 
+    #[cfg(all())]
     fn __isub__(&mut self, rhs: RaggedBufferF32) -> PyResult<()> {
         self.0.binop_mut::<crate::ragged_buffer::BinOpSub>(&rhs.0)
     }
-}
 
-#[pyproto]
-impl<'p> PyMappingProtocol for RaggedBufferF32 {
-    fn __getitem__(&self, index: MultiIndex<'p>) -> PyResult<RaggedBufferF32> {
+    fn __getitem__(&self, index: MultiIndex) -> PyResult<RaggedBufferF32> {
         match index {
             MultiIndex::Index1(index) => match index {
                 Index::PermutationNP(indices) => Ok(RaggedBufferF32(self.0.swizzle(indices)?)),
@@ -188,3 +184,9 @@ impl<'p> PyMappingProtocol for RaggedBufferF32 {
         self.0.len()
     }
 }
+
+#[derive(FromPyObject)]
+pub enum RaggedBufferF32OrF32<'p> {
+    RB(PyRef<'p, RaggedBufferF32>),
+    Scalar(f32),
+}
